# IBM WebSphere Application Server Patterns
#
#  Copyright IBM Corp.  2015
#  All rights reserved.
#  US Government Users Restricted Rights -
#  Use, duplication or disclosure restricted
#  by GSA ADP Schedule Contract with IBM Corp.
#
#  Licensed Materials-Property of IBM
#
#--------------------------------------------------------------------
# WebSphere Tuning Script for ODR
#--------------------------------------------------------------------
#
# This script is designed to modify some of the most common
# WebSphere configuration parameters and tuning knobs.
# In order to tune the config parameters, simply change the values
# provided below.
#
# To invoke the script, type:
#   wsadmin -f tuneODRProfile.py <node_name> <server_name> <jvm_heap_size>
#      node_name      - Name of the node, i.e. hostnameNode01
#      server_name    - Name of the server, i.e. odr
#      jvm_heap_size  - Specifies the Java heap size (in MB)
#
# Example:
#   wsadmin -f tuneODRProfile.py hostnameNode01Cell hostnameNode01 odr 1024
#
#--------------------------------------------------------------------
AdminConfig.setValidationLevel("NONE" )
from math import log as _log, ceil as _ceil, pow as _pow
from sys import registry
print "Starting script..."
print "Reading config parameters..."
#--------------------------------------------------------------------
# COMMON CONFIG PARAMETERS
# - Adjust these parameters based on the intended target system
#--------------------------------------------------------------------
# Initial JVM heap size properties, they will be replaced by parm passed and adjustments
minHeap = 256
maxHeap = 256
genericArgs = "-Dcom.ibm.tools.attach.enable=no"
#---------------------------------------------
# Check/Print Usage
#---------------------------------------------
def printUsageAndExit (  ):
        print " "
        print "Usage: wsadmin -f tuneODRProfile.py <node_name> <server_name> <jvm_heap_size>"
        sys.exit()
#endDef
#---------------------------------------------
# Misc Procedures
#---------------------------------------------
def getName (objectId):
        endIndex = (objectId.find("(c") - 1)
        stIndex = 0
        if (objectId.find("\"") == 0):
                stIndex = 1
        #endIf
        return objectId[stIndex:endIndex+1]
#endDef
#---------------------------------------------
# Parse command line arguments
#---------------------------------------------
print ""
print "Command line arguments"
print "----------------------"
if (len(sys.argv) < 3):
        printUsageAndExit( )
else:
    #parse the parameters from command line
    nodeName = sys.argv[0];
    serverName = sys.argv[1]
    #adjust memory info
    minHeap = int(sys.argv[2])
    multFactor = 8
    lastMb = 0
    while ( lastMb < minHeap ):
        lastMb = lastMb + multFactor

    #use a multiple of "multFactor"
    if ( minHeap != lastMb ):
        lastMb = lastMb - multFactor
        print "*Using "+`lastMb`+"MB instead of "+`minHeap`+"MB"
        print ""
        minHeap = lastMb
    #endif
    minHeap = str(minHeap)
    maxHeap = minHeap
    #print all values
    print "Node:       "+`nodeName`
    print "Server:     "+`serverName`
    print "Heap Size:  "+`minHeap`
    #endElse
#endElse
#---------------------------------------------
# Obtain server list
#---------------------------------------------
print ""
print "Locating server configuration..."
print "--------------------------------"

server = AdminConfig.getid('/Cell:/Node:%s/Server:%s/' % (nodeName, serverName))
if (not server):
    print "Configuration for /Cell/"+nodeName+"/"+serverName+" not found!"
    print "Exiting..."
    sys.exit(0)
else:
    print "Success location server config!"
#endif

#---------------------------------------------
# Print config properties
#---------------------------------------------
print ""
print "WebSphere configuration"
print "-----------------------"
print "Servers:"
print "   " + getName(server)
print ""
print " JVM --------------------------------------------"
print "   Min JVM Heap Size:           "+str(minHeap)
print "   Max JVM Heap Size:           "+str(maxHeap)
print "   Generic JVM Arguments:  "
print "      "+genericArgs
print ""
#---------------------------------------------
# Modify cell parameters
#---------------------------------------------
#---------------------------------------------
# Modify server parameters
#---------------------------------------------
servername = getName(server )
print ""
print "Server: "+servername
print ""

# Accessing JVM config
print "Accessing JVM configuration..."
jvm = AdminConfig.list("JavaVirtualMachine", server )
print "Initial Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "initialHeapSize")+"/"+str(minHeap)
print "Maximum Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "maximumHeapSize")+"/"+str(maxHeap)
newJvmArg = AdminConfig.showAttribute(jvm, "genericJvmArguments")+" "+genericArgs
print ""
print "New General JVM Arguments"
print  newJvmArg
attrs = [["initialHeapSize", minHeap], ["maximumHeapSize", maxHeap], ["genericJvmArguments", newJvmArg]]
print "Updating JVM..."
print " "
AdminConfig.modify(jvm, attrs )

# Update proxy settings
print "Accessing proxy configuration..."
proxy_settings_id = AdminConfig.getid( '/Node:%s/Server:%s/ProxySettings:/' % ( nodeName, serverName ) )
print "Enable proxy caching (old/new):  "+AdminConfig.showAttribute(proxy_settings_id, "enableCaching")+"/false"
print "Updating proxy..."
AdminConfig.modify( proxy_settings_id, [["enableCaching", "false"]])

print ""
print "Saving config..."
AdminConfig.save()
print "Done"

print " "
print "Synchronizing changes on the ODR node..."
Sync1 = AdminControl.completeObjectName( "type=NodeSync,node=%s,*" % nodeName )
AdminControl.invoke( Sync1, 'sync' )
print "Done"


