# IBM WebSphere Application Server Patterns
#
#  Copyright IBM Corp. 2015
#  All rights reserved.
#  US Government Users Restricted Rights -
#  Use, duplication or disclosure restricted
#  by GSA ADP Schedule Contract with IBM Corp.
#
#  Licensed Materials-Property of IBM
#
#--------------------------------------------------------------------
# WebSphere Tuning Script for Nodeagent/Custom Node App Servers
#--------------------------------------------------------------------
#
# This script is designed to modify some of the most common
# WebSphere configuration parameters and tuning knobs.
# In order to tune the config parameters, simply change the values
# provided below.
#
# To invoke the script, type:
#   wsadmin -f tuneOtherJVM.py <node_name>  <amt_of_memory> <num_of_cps>
#      node_name      - Name of the node, i.e. hostnameNode01
#      amt_mem        - Amount of Memory in the system
#      num_cpu        - Number of CPUs in the system
#      min_ram        - Minimum amount of memory
#      max_ram        - Maximun amount of memory
#      os_ram         - Amount of memory overhead
#      threadsPerCPU  - Number of threads per cpu
#
# Example:
#   wsadmin -f tuneOtherJVM.py hostnameNode01 4049108 1 256 4096 1280 10
#
#--------------------------------------------------------------------
AdminConfig.setValidationLevel("NONE" )
from math import log as _log, ceil as _ceil, pow as _pow
from sys import registry
print "Starting script..."
print "Reading config parameters..."
#--------------------------------------------------------------------
# COMMON CONFIG PARAMETERS
# - Adjust these parameters based on the intended target system
#--------------------------------------------------------------------
#minRam=256
#maxRam=4096
#osRam=1280
#threadsPerCPU=10
#
heapsize=1024
minWebPool = 50
maxWebPool = 50
# JVM properties
minHeap = 256
maxHeap = 256
# JVM properties
genericArgs = "-Dcom.ibm.tools.attach.enable=no" 
#genericJvmArguments = "-Dcom.ibm.tools.attach.enable=no -Dcom.ibm.cacheLocalHost=true -Xcodecache20m -Xlp64k"
genericJvmArguments = "-Dcom.ibm.tools.attach.enable=no -Dcom.ibm.cacheLocalHost=true -Xcodecache20m"
#---------------------------------------------
# Check/Print Usage
#---------------------------------------------
def printUsageAndExit (  ):
        print " "
        print "Usage: wsadmin -f tuneOtherJVM.py <node_name> <amt_mem> <num_cpu> <min_ram> <max_ram> <os_ram> <thread_per_cpu>"
        sys.exit()
#endDef
#---------------------------------------------
# Misc Procedures
#---------------------------------------------
def getName (objectId):
        endIndex = (objectId.find("(c") - 1)
        stIndex = 0
        if (objectId.find("\"") == 0):
                stIndex = 1
        #endIf
        return objectId[stIndex:endIndex+1]
#endDef

def convertToList(inlist):
     outlist=[]
     if (len(inlist)>0 and inlist[0]=='[' and inlist[len(inlist)-1]==']'):
        inlist = inlist[1:len(inlist)-1]
        clist = inlist.split(" ")
     else:
        clist = inlist.split(lineSeparator)
     for elem in clist:
        elem=elem.rstrip();
        if (len(elem)>0):
           outlist.append(elem)
     return outlist
#endDef
#---------------------------------------------
# Parse command line arguments
#---------------------------------------------
print ""
print "Command line arguments"
print "----------------------"
if (len(sys.argv) < 7):
        printUsageAndExit( )
else:
    #parse the parameters from command line
    nodeName = sys.argv[0];
    amtMem = int(sys.argv[1])
    numCPU = int(sys.argv[2])
    minRam = int(sys.argv[3])
    maxRam = int(sys.argv[4])
    osRam  = int(sys.argv[5])
    threadsPerCPU = int(sys.argv[6])
      
    #print all values
    print "Node:       "+`nodeName`
    print "Amount of Memory: "+`amtMem`
    print "Number of CPUs: "+`numCPU`
    print "Min amount of memory: "+`minRam`
    print "Max amount of memory: "+`maxRam`
    print "Amount of memory overhead: "+`osRam`
    print "Number of threads per CPU: "+`threadsPerCPU`
    #endElse
#endElse
#---------------------------------------------
# Find # of Servers in the Node
#---------------------------------------------
numOfServers=0
serverNames=[]
node = AdminConfig.getid('/Cell:/Node:%s' % (nodeName))
serverEntries = convertToList(AdminConfig.list("ServerEntry",node))
for serverEntry in serverEntries:
	serverType = AdminConfig.showAttribute(serverEntry,"serverType")
	if (serverType=="APPLICATION_SERVER"):
		serverName = AdminConfig.showAttribute(serverEntry,"serverName")
		serverNames.append(serverName)       
	#endIf
#endFor

# If num of server is 0 then there is only nodeagent in the custom node.
numOfServers=len(serverNames)
print "Number of Servers: "+`numOfServers`

if (numOfServers!=0):
	#---------------------------------------------
	# Calculate numCPU,  heapsize, threads
	#---------------------------------------------
	threads = threadsPerCPU*numCPU

	if (numOfServers==1):
		heapsize= (amtMem/1024)-osRam
	else:
		heapsize = ((amtMem/1024)-osRam)/numOfServers
		threads = threads/numOfServers
	#endif
	if (heapsize < minRam):
		print "Heap size does not meet the minimum size requirement: "+`heapsize`
 		heapsize = minRam
		sys.exit()
	#endif
	if (heapsize > maxRam):
   		heapsize = maxRam
	#endif

	minWebPool = threads
	maxWebPool = minWebPool
	minHeap = heapsize
	multFactor = 8
	lastMb = 0
	while ( lastMb < minHeap ):
		lastMb = lastMb + multFactor
	
	#use a multiple of "multFactor"
	if ( minHeap != lastMb ):
		lastMb = lastMb - multFactor
		print "*Using "+`lastMb`+"MB instead of "+`minHeap`+"MB"
		print ""
		minHeap = lastMb
	#endif
	minHeap = str(minHeap)
	maxHeap = minHeap

	print "Threads:    "+`minWebPool`
	print "Heap Size:  "+`minHeap`
#endif
#---------------------------------------------
# Looping thru all Servers to be Tuned
#---------------------------------------------
nodeId = AdminConfig.getid('/Cell:/Node:%s/' % (nodeName))
serverEntries = convertToList(AdminConfig.list("ServerEntry",nodeId))
for serverEntry in serverEntries: 
		serverType = AdminConfig.showAttribute(serverEntry,"serverType")       
		if (serverType=="APPLICATION_SERVER"):	
			serverName = AdminConfig.showAttribute(serverEntry,"serverName")
			server = AdminConfig.getid('/Cell:/Node:%s/Server:%s/' % (nodeName, serverName))
			#---------------------------------------------
			# Print config properties
			#---------------------------------------------
			print ""
			print "WebSphere configuration"
			print "-----------------------"
			print "Server:"
			print "   " + getName(server)
			print ""
			print " Web --------------------------------------------"
			print "   Min WebContainer Pool Size:  "+str(minWebPool)
			print "   Max WebContainer Pool Size:  "+str(maxWebPool)
			print " JVM --------------------------------------------"
			print "   Min JVM Heap Size:           "+str(minHeap)
			print "   Max JVM Heap Size:           "+str(maxHeap)
			print "   Generic JVM Arguments:  "
			print "      "+genericJvmArguments
			print ""
			#---------------------------------------------
			# Modify server parameters
			#---------------------------------------------
			servername = getName(server )
			print ""
			print "Server: "+servername
			print ""
			# Accessing web container thread pool config
			print "Accessing web container configuration..."
			tpList = AdminConfig.list("ThreadPool", server )
			webPool = ""
			for pool in tpList.split("\n"):
				pool = pool.rstrip()
				if (getName(pool) == "WebContainer"):
					webPool = pool
					break
				#endIf
			#endFor
			print "ThreadPool MaxSize (old/new):             "+AdminConfig.showAttribute(webPool, "maximumSize")+"/"+str(maxWebPool)
			print "ThreadPool MinSize (old/new):             "+AdminConfig.showAttribute(webPool, "minimumSize")+"/"+str(minWebPool)
			attrs = [["maximumSize", maxWebPool], ["minimumSize", minWebPool]]
			print "Updating web container..."
			print " "
			AdminConfig.modify(webPool, attrs )
			# Accessing JVM config
			print "Accessing JVM configuration..."
			jvm = AdminConfig.list("JavaVirtualMachine", server )
			print "Initial Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "initialHeapSize")+"/"+str(minHeap)
			print "Maximum Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "maximumHeapSize")+"/"+str(maxHeap)
			print "General JVM Arguments (old/new):  "+str(AdminConfig.showAttribute(jvm, "genericJvmArguments"))+"/%s" % (genericJvmArguments)
			newJvmArg = AdminConfig.showAttribute(jvm, "genericJvmArguments")+" "+genericJvmArguments
			print ""
			print "New General JVM Arguments"
			print  newJvmArg
			attrs = [["initialHeapSize", minHeap], ["maximumHeapSize", maxHeap], ["genericJvmArguments", newJvmArg]]
			print "Updating JVM..."
			print " "
			AdminConfig.modify(jvm, attrs )
			print ""
			print "completed...updating JVM"
			print "Saving config..."
			AdminConfig.save()	  
		#endif		
		if (serverType=="NODE_AGENT"):	
			serverName = AdminConfig.showAttribute(serverEntry,"serverName")
			server = AdminConfig.getid('/Cell:/Node:%s/Server:%s/' % (nodeName, serverName)) 
				
			#---------------------------------------------
			# Print config properties
			#---------------------------------------------
			print ""
			print "WebSphere configuration"
			print "-----------------------"
			print "Server:"
			print "   " + getName(server)
			print ""
			print " JVM --------------------------------------------"
			print "   Generic JVM Arguments:  "
			print "      "+genericArgs
			print ""
			#---------------------------------------------
			# Modify cell parameters
			#---------------------------------------------
			#
			# Modify server parameters
			#---------------------------------------------
			servername = getName(server )
			print ""
			print "Server: "+servername
			print ""
			# Accessing JVM config
			print "Accessing JVM configuration..."
			jvm = AdminConfig.list("JavaVirtualMachine", server )
			newJvmArg = AdminConfig.showAttribute(jvm, "genericJvmArguments")+" "+genericArgs
			print ""
			print "New General JVM Arguments"
			print  newJvmArg
			attrs = [["genericJvmArguments", newJvmArg]]	
			print "Updating JVM..."
			print " "
			AdminConfig.modify(jvm, attrs )
			print ""
			print "Completed updating JVM ..."
			print "Saving config..."
			AdminConfig.save()
        #endIF  
 #endFor

