# IBM WebSphere Application Server Patterns
#
#  Copyright IBM Corp. 2015
#  All rights reserved.
#  US Government Users Restricted Rights -
#  Use, duplication or disclosure restricted
#  by GSA ADP Schedule Contract with IBM Corp.
#
#  Licensed Materials-Property of IBM
#
#--------------------------------------------------------------------
# WebSphere Tuning Script
#--------------------------------------------------------------------
#
# This script is designed to modify some of the most common
# WebSphere configuration parameters and tuning knobs.
# In order to tune the config parameters, simply change the values
# provided below.
#
# To invoke the script, type:
#   wsadmin -f tuneWASProfile.py <node_name> <server_name> <web_container_threads> <jvm_heap_size>
#      node_name      - Name of the node, i.e. hostnameNode01
#      server_name    - Name of the server, i.e. server1
#      num_threads    - Specifies number of web container threads to use for WAS tuning
#      jvm_heap_size  - Specifies the Java heap size (in MB)
#
# Example:
#   wsadmin -f tuneWASProfile.py hostnameNode01 server1 10 1024
#
#--------------------------------------------------------------------
AdminConfig.setValidationLevel("NONE" )
from math import log as _log, ceil as _ceil, pow as _pow
from sys import registry
print "Starting script..."
print "Reading config parameters..."
#--------------------------------------------------------------------
# COMMON CONFIG PARAMETERS
# - Adjust these parameters based on the intended target system
#--------------------------------------------------------------------
# WebContainer Thread Pool initial values(50,50), they will be replaced by parms passed 
minWebPool = 50
maxWebPool = 50
# Initial JVM heap size properties, they will be replaced by parm passed and adjustments
minHeap = 256
maxHeap = 256
#genericJvmArguments = "-Dcom.ibm.tools.attach.enable=no -Dcom.ibm.cacheLocalHost=true -Xcodecache20m -Xlp64k"
genericJvmArguments = "-Dcom.ibm.tools.attach.enable=no -Dcom.ibm.cacheLocalHost=true -Xcodecache20m"
#---------------------------------------------
# Check/Print Usage
#---------------------------------------------
def printUsageAndExit (  ):
        print " "
        print "Usage: wsadmin -f tuneWASProfile.py <node_name> <server_name> <web_container_threads> <jvm_heap_size>"
        sys.exit()
#endDef
#---------------------------------------------
# Misc Procedures
#---------------------------------------------
def getName (objectId):
        endIndex = (objectId.find("(c") - 1)
        stIndex = 0
        if (objectId.find("\"") == 0):
                stIndex = 1
        #endIf
        return objectId[stIndex:endIndex+1]
#endDef
#---------------------------------------------
# Parse command line arguments
#---------------------------------------------
print ""
print "Command line arguments"
print "----------------------"
if (len(sys.argv) < 4):
        printUsageAndExit( )
else:
    #parse the parameters from command line
    nodeName = sys.argv[0];
    serverName = sys.argv[1]
    minWebPool = sys.argv[2]
    maxWebPool = minWebPool
    #adjust memory info
    minHeap = int(sys.argv[3])
    multFactor = 8
    lastMb = 0
    while ( lastMb < minHeap ):
        lastMb = lastMb + multFactor

    #use a multiple of "multFactor"
    if ( minHeap != lastMb ):
        lastMb = lastMb - multFactor
        print "*Using "+`lastMb`+"MB instead of "+`minHeap`+"MB"
        print ""
        minHeap = lastMb
    #endif
    minHeap = str(minHeap)
    maxHeap = minHeap
    #print all values
    print "Node:       "+`nodeName`
    print "Server:     "+`serverName`
    print "Threads:    "+`minWebPool`
    print "Heap Size:  "+`minHeap`
    #endElse
#endElse
#---------------------------------------------
# Obtain server list
#---------------------------------------------
print ""
print "Locating server configuration..."
print "--------------------------------"

server = AdminConfig.getid('/Cell:/Node:%s/Server:%s/' % (nodeName, serverName))
if (not server):
    print "Configuration for /Cell/"+nodeName+"/"+serverName+" not found!"
    print "Exiting..."
    sys.exit(0)
else:
    print "Success location server config!"
#endif

#---------------------------------------------
# Print config properties
#---------------------------------------------
print ""
print "WebSphere configuration"
print "-----------------------"
print "Servers:"
print "   " + getName(server)
print ""
print " Web --------------------------------------------"
print "   Min WebContainer Pool Size:  "+str(minWebPool)
print "   Max WebContainer Pool Size:  "+str(maxWebPool)
print " JVM --------------------------------------------"
print "   Min JVM Heap Size:           "+str(minHeap)
print "   Max JVM Heap Size:           "+str(maxHeap)
print "   Generic JVM Arguments:  "
print "      "+genericJvmArguments
print ""
#---------------------------------------------
# Modify server parameters
#---------------------------------------------
servername = getName(server )
print ""
print "Server: "+servername
print ""
# Accessing web container thread pool config
print "Accessing web container configuration..."
tpList = AdminConfig.list("ThreadPool", server )
webPool = ""
for pool in tpList.split("\n"):
    pool = pool.rstrip()
    if (getName(pool) == "WebContainer"):
        webPool = pool
        break
        #endIf
#endFor
print "ThreadPool MaxSize (old/new):             "+AdminConfig.showAttribute(webPool, "maximumSize")+"/"+str(maxWebPool)
print "ThreadPool MinSize (old/new):             "+AdminConfig.showAttribute(webPool, "minimumSize")+"/"+str(minWebPool)
attrs = [["maximumSize", maxWebPool], ["minimumSize", minWebPool]]
print "Updating web container..."
print " "
AdminConfig.modify(webPool, attrs )
# Accessing JVM config
print "Accessing JVM configuration..."
jvm = AdminConfig.list("JavaVirtualMachine", server )
print "Initial Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "initialHeapSize")+"/"+str(minHeap)
print "Maximum Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "maximumHeapSize")+"/"+str(maxHeap)
print "General JVM Arguments (old/new):  "+str(AdminConfig.showAttribute(jvm, "genericJvmArguments"))+"/%s" % (genericJvmArguments)
newJvmArg = AdminConfig.showAttribute(jvm, "genericJvmArguments")+" "+genericJvmArguments
print ""
print "New General JVM Arguments"
print  newJvmArg
attrs = [["initialHeapSize", minHeap], ["maximumHeapSize", maxHeap], ["genericJvmArguments", newJvmArg]]
print "Updating JVM..."
print " "
AdminConfig.modify(jvm, attrs )

print ""
print "Script completed..."
print "Saving config..."
AdminConfig.save()
